// Copyright (c) 2013-2015  Mario Albert
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"
#include <bitset>
using namespace std;
using namespace CoCoA;
using namespace CoCoA::Involutive;

//----------------------------------------------------------------------
const string ShortDescription =
  "In this file we explain how to compute the Janet Basis in CoCoA.  \n";

const string LongDescription =
  " We explain the function JanetBasis and show whicht options we can choose. \n";
//----------------------------------------------------------------------

void output(vector<RingElem> vec)
{
  for(vector<RingElem>::iterator iter = vec.begin(); iter != vec.end(); ++iter)
    {
      cout << *iter << endl;
    }
}


void program()
{
  GlobalManager CoCoAFoundations;

  cout << ShortDescription << endl;
  ring Q = RingQQ();

  cout <<"/////////////////////////////////////Cyclic 5//////////////////////////////////////////" << endl;
  SparsePolyRing polyRingCyc5 = NewPolyRing(Q, SymbolRange("x",0,4), StdDegLex);
  const vector<RingElem> x = indets(polyRingCyc5);
  vector<RingElem>cyclic5;
  cyclic5.push_back(x[0]+x[1]+x[2]+x[3]+x[4]);
  cyclic5.push_back(x[0]*x[1] + x[2]*x[1] + x[2]*x[3] + x[3]*x[4] + x[0]*x[4]);
  cyclic5.push_back(x[1]*x[2]*x[0] + x[2]*x[3]*x[1] + x[4]*x[2]*x[3] + x[0]*x[4]*x[1] + x[0]*x[4]*x[3]);
  cyclic5.push_back(x[0]*x[1]*x[2]*x[3]+x[0]*x[1]*x[2]*x[4]+x[0]*x[1]*x[3]*x[4]+x[0]*x[2]*x[3]*x[4]+x[2]*x[3]*x[1]*x[4]);
  cyclic5.push_back(x[0]*x[1]*x[2]*x[3]*x[4] -1 );
  ideal I = ideal(cyclic5);

  cout << "---------computing the Janet Base with a list of polynomials(TQBlockLow):------------------------" << endl;
  cout << "-----------------------------first two criteria ---------------------------------" << endl;
  Involutive::JBMill::Builder builder;

  builder.setInput(cyclic5);
  builder.setStrategy(Involutive::TQBlockLow);
  builder.setInvolutiveCriteria(bitset<3>(string("110")));

  Involutive::JBMill mill1(builder);


  cout << "-----------------------------Output GB ---------------------------------" << endl;
  vector<RingElem> result(mill1.myReturnGB());
  std::cout <<  "result = " << result << std::endl;
  std::cout << "len(result) = " << len(result) << std::endl;

  cout << "-----------------------------Output JB---------------------------------" << endl;
  result = mill1.myReturnJB();
  std::cout <<  "result = " << result << std::endl;
  std::cout << "len(result) = " << len(result) << std::endl;
}

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    cerr << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(cerr, err);
  }
  catch (const std::exception& exc)
  {
    cerr << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    cerr << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}
